==> virtualfs-1.23.src.tar.gz: virtualfs-1.23/browse/virtualfs-browsenet.cc <==
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include "gnomba.h"
#include "../remaudio.h"

machineNode * netscan(const char *range[], int nbrange);
void net_delnode (machineNode *root);


static int ifconfig_ioctl(
	int fd,
	const char *ifname,
	int cmd,
	struct ifreq &ifr)
{
	strcpy(ifr.ifr_name, ifname);
	return ioctl(fd, cmd,&ifr);
}

static unsigned long ifconfig_cnv (struct sockaddr &adr)
{
	struct sockaddr_in *sin = (struct sockaddr_in*)&adr;
	return htonl(sin->sin_addr.s_addr);
}


/*
	Fetch the inteface configuration from the kernel.
	Assume the device is already available in the kernel
	Return -1 if any error.
*/
static int ifconfig_getinfo (
	const char *ifname,
	char *addr,		// IP address of the device
	char *range)	// Range of the network (first IP available -> last)
{
	int ret = -1;
	addr[0] = '\0';
	range[0] = '\0';
	int skfd = socket(AF_INET, SOCK_DGRAM, 0);
	if (skfd != -1){
		struct ifreq ifr;
		if (ifconfig_ioctl(skfd, ifname, SIOCGIFFLAGS, ifr) >= 0){
			ret = 0;
			// In linux 2.1, unconfigured interfaces fail for the
			// following calls, while with 2.0 and below, they
			// return 0 values.
			struct ifreq ifr_mask;
			if (ifconfig_ioctl(skfd,ifname,SIOCGIFADDR, ifr) >= 0
				&& ifconfig_ioctl(skfd, ifname,SIOCGIFNETMASK, ifr_mask) >= 0) {
				unsigned long iface_addr = ifconfig_cnv(ifr.ifr_addr);
				unsigned long mask = ifconfig_cnv(ifr_mask.ifr_netmask);
				unsigned long net = iface_addr & mask;
				unsigned long bcast = net | (~mask);
				__ipnum_ip2a (iface_addr,addr);
				char start[20],end[20];
				__ipnum_ip2a (net+1,start);
				__ipnum_ip2a (bcast-1,end);
				sprintf (range,"%s %s",start,end);
				ret = 0;
			}
		}
		close (skfd);
	}
	return ret;
}

static machineNode *v_readcache(FILE *fin)
{
	machineNode *ret = NULL;
	char group[PATH_MAX],name[PATH_MAX];
	while (fscanf(fin,"%s\t%s\n",group,name)==2){
		machineNode *node = new machineNode;
		node->workgroupName = strdup(group);
		node->machineName = strdup(name);
		node->next = ret;
		ret = node;
	}
	return ret;
}


static machineNode *v_scan ()
{
	machineNode *ret = NULL;
	char range[40],addr[20],cachefile[PATH_MAX];
	bool info_available = ifconfig_getinfo ("eth0",addr,range) !=  -1;
	const char *tb[100];
	int nb = 0;
	const char *home = getenv ("HOME");
	cachefile[0] = '\0';
	if (home != NULL){
		/* #Specification: browsenet / configuration file and cache
			The browse system supports a solution for notebook.
			The utility is looking for a configuration file in ~/.virtualfs.
			It first look for browsenet-IP_ADDR.conf. If missing, it is looking
			for browsenet.conf. If also missing, it is looking for
			/etc/virtualfs/browsenet.conf. If missing, it is using the spec
			of the ETH0 adaptor to retrieve the range of the browse.

			The utility is producing also a cache copy of the result. It is
			reused until erased. The cache file is stored in
			~/.virtualfs/browsenet-IP_ADDR.cache.

			IP_ADDR is the IP address of the eth0 adaptor.
		*/
		snprintf (cachefile,sizeof(cachefile)-1
			,"%s/.virtualfs/browsenet-%s.cache",home,addr);
		FILE *fin = fopen (cachefile,"r");
		if (fin != NULL){
			ret = v_readcache(fin);
			fclose (fin);
		}else{
			char path[PATH_MAX];
			snprintf (path,sizeof(path)-1,"%s/.virtualfs/browsenet-%s.conf",home
				,addr);
			fin = fopen (path,"r");
			if (fin == NULL){
				snprintf (path,sizeof(path)-1,"%s/.virtualfs/browsenet.conf",home);
				fin = fopen (path,"r");
			}
			if (fin == NULL){
				fin = fopen ("/etc/browsenet.conf","r");
			}
			if (fin != NULL){
				char buf[1000];
				while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
					if (buf[0] != '#') tb[nb++] = strdup(buf);
				}
				fclose (fin);
			}
		}
	}
	if (ret == NULL){
		if (nb == 0 && info_available){
			tb[nb++] = strdup(range);
		}
		if (nb == 0){
			fprintf (stderr,"Neither file /etc/browsenet.conf nor ~/.virtualfs/browsenet.conf exist\n");
			fprintf (stderr,"\tCan't browse\n");
		}
		ret = nb > 0 ? netscan (tb,nb) : NULL;
		if (ret != NULL && cachefile[0] != '\0'){
			// Update the cache
			FILE *fout = fopen (cachefile,"w");
			if (fout == NULL){
				fprintf (stderr,"Can't open cache file %s (%s)\n",cachefile
					,strerror (errno));
			}else{
				machineNode *pt = ret;
				while (pt != NULL){
					fprintf (fout,"%s\t%s\n",pt->workgroupName
						,pt->machineName);
					pt = pt->next;
				}
				fclose (fout);
			}
		}
	}
	for (int i=0; i<nb; i++) free ((char*)tb[i]);
	return ret;
}


int main (int argc, char *argv[])
{
	if ((argc == 2 || argc == 3) && strcmp(argv[1],"--list")==0){
		uid_t uid = getuid();
		gid_t gid = getgid();
		if (argc == 2){
			// we want to full map group/machine
			// litefsd-browse is smart enough to organise the info
			// in directory and subdirectory
			machineNode *root = v_scan ();
			printf ("t 10\n");
			machineNode *pt = root;
			while (pt != NULL){
				printf ("d %s/%s %d %d 1000\n",pt->workgroupName
					,pt->machineName,uid,gid);
				pt = pt->next;
			}
			net_delnode (root);
		}else if (argc == 3){
			char group[PATH_MAX];
			strncpy (group,argv[2],sizeof(group)-1);
			group[sizeof(group)-1] = '\0';
			char *pt = strchr(group,'/');
			if (pt == NULL){
				// We want the machine member of a work group
				machineNode *root = v_scan ();
				printf ("t 300\n");
				machineNode *pt = root;
				while (pt != NULL){
					if (strcasecmp(pt->workgroupName,group)==0){
						printf ("d %s %d %d 1000\n",pt->machineName,uid,gid);
					}
					pt = pt->next;
				}
				net_delnode (root);
			}else{
				*pt++ = '\0';
				char cmd[PATH_MAX];
				snprintf (cmd,sizeof(cmd)-1,"smbclient -W \"%s\" -L \"%s\""
					,group,pt);
				int fd = open ("/dev/tty",O_RDONLY);
				if (fd != -1){
					ioctl (fd,TIOCNOTTY);
					close (fd);
				}
				int tbpipe[2];
				if (pipe(tbpipe)!=-1){
					int oldout = dup(1);
					dup2 (tbpipe[1],1);
					close (tbpipe[1]);
					int err = open ("/dev/null",O_WRONLY);
					dup2 (err,2);
					close (err);
					FILE *fout = popen (cmd,"w");
					if (fout != NULL){
						dup2 (oldout,1);
						close (oldout);
						fputs ("\n",fout);
						fflush (fout);
						FILE *fin = fdopen (tbpipe[0],"r");
						char buf[1000];
						int state = 0;
						printf ("t 30\n");
						while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
							char *pt = buf;
							while (isspace (*pt)) pt++;
							if (strncmp(pt,"---------",9)==0){
								state ++;
							}else if (state == 1){
								if (pt[0] == '\0') break;
								char *start = pt;
								while (*pt > ' ') pt++;
								*pt = '\0';
								printf ("d %s %d %d 1000\n",start,uid,gid);
							}
						}
						fclose (fin);
						close (tbpipe[0]);
						pclose (fout);
					}
				}
			}
		}
	}
	return 0;
}

==> virtualfs-1.24.src.tar.gz: virtualfs-1.24/browse/virtualfs-browsenet.cc <==
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include "gnomba.h"
#include "../remaudio.h"

machineNode * netscan(const char *range[], int nbrange);
void net_delnode (machineNode *root);


static int ifconfig_ioctl(
	int fd,
	const char *ifname,
	int cmd,
	struct ifreq &ifr)
{
	strcpy(ifr.ifr_name, ifname);
	return ioctl(fd, cmd,&ifr);
}

static unsigned long ifconfig_cnv (struct sockaddr &adr)
{
	struct sockaddr_in *sin = (struct sockaddr_in*)&adr;
	return htonl(sin->sin_addr.s_addr);
}


/*
	Fetch the inteface configuration from the kernel.
	Assume the device is already available in the kernel
	Return -1 if any error.
*/
static int ifconfig_getinfo (
	const char *ifname,
	char *addr,		// IP address of the device
	char *range)	// Range of the network (first IP available -> last)
{
	int ret = -1;
	addr[0] = '\0';
	range[0] = '\0';
	int skfd = socket(AF_INET, SOCK_DGRAM, 0);
	if (skfd != -1){
		struct ifreq ifr;
		if (ifconfig_ioctl(skfd, ifname, SIOCGIFFLAGS, ifr) >= 0){
			ret = 0;
			// In linux 2.1, unconfigured interfaces fail for the
			// following calls, while with 2.0 and below, they
			// return 0 values.
			struct ifreq ifr_mask;
			if (ifconfig_ioctl(skfd,ifname,SIOCGIFADDR, ifr) >= 0
				&& ifconfig_ioctl(skfd, ifname,SIOCGIFNETMASK, ifr_mask) >= 0) {
				unsigned long iface_addr = ifconfig_cnv(ifr.ifr_addr);
				unsigned long mask = ifconfig_cnv(ifr_mask.ifr_netmask);
				unsigned long net = iface_addr & mask;
				unsigned long bcast = net | (~mask);
				__ipnum_ip2a (iface_addr,addr);
				char start[20],end[20];
				__ipnum_ip2a (net+1,start);
				__ipnum_ip2a (bcast-1,end);
				sprintf (range,"%s %s",start,end);
				ret = 0;
			}
		}
		close (skfd);
	}
	return ret;
}

static machineNode *v_readcache(FILE *fin)
{
	machineNode *ret = NULL;
	char group[PATH_MAX],name[PATH_MAX];
	while (fscanf(fin,"%s\t%s\n",group,name)==2){
		machineNode *node = new machineNode;
		node->workgroupName = strdup(group);
		node->machineName = strdup(name);
		node->next = ret;
		ret = node;
	}
	return ret;
}


static machineNode *v_scan ()
{
	machineNode *ret = NULL;
	char range[40],addr[20],cachefile[PATH_MAX];
	bool info_available = ifconfig_getinfo ("eth0",addr,range) !=  -1;
	const char *tb[100];
	int nb = 0;
	const char *home = getenv ("HOME");
	cachefile[0] = '\0';
	if (home != NULL){
		/* #Specification: browsenet / configuration file and cache
			The browse system supports a solution for notebook.
			The utility is looking for a configuration file in ~/.virtualfs.
			It first look for browsenet-IP_ADDR.conf. If missing, it is looking
			for browsenet.conf. If also missing, it is looking for
			/etc/virtualfs/browsenet.conf. If missing, it is using the spec
			of the ETH0 adaptor to retrieve the range of the browse.

			The utility is producing also a cache copy of the result. It is
			reused until erased. The cache file is stored in
			~/.virtualfs/browsenet-IP_ADDR.cache.

			IP_ADDR is the IP address of the eth0 adaptor.
		*/
		snprintf (cachefile,sizeof(cachefile)-1
			,"%s/.virtualfs/browsenet-%s.cache",home,addr);
		FILE *fin = fopen (cachefile,"r");
		if (fin != NULL){
			ret = v_readcache(fin);
			fclose (fin);
		}else{
			char path[PATH_MAX];
			snprintf (path,sizeof(path)-1,"%s/.virtualfs/browsenet-%s.conf",home
				,addr);
			fin = fopen (path,"r");
			if (fin == NULL){
				snprintf (path,sizeof(path)-1,"%s/.virtualfs/browsenet.conf",home);
				fin = fopen (path,"r");
			}
			if (fin == NULL){
				fin = fopen ("/etc/browsenet.conf","r");
			}
			if (fin != NULL){
				char buf[1000];
				while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
					if (buf[0] != '#') tb[nb++] = strdup(buf);
				}
				fclose (fin);
			}
		}
	}
	if (ret == NULL){
		if (nb == 0 && info_available){
			tb[nb++] = strdup(range);
		}
		if (nb == 0){
			fprintf (stderr,"\n");
			fprintf (stderr,"*** Neither file /etc/browsenet.conf nor ~/.virtualfs/browsenet.conf exist\n");
			fprintf (stderr,"*** Can't find info about ETH0 network adaptor either.\n");
			fprintf (stderr,"*** Can't browse\n");
			fprintf (stderr,"\n");
		}
		ret = nb > 0 ? netscan (tb,nb) : NULL;
		if (ret != NULL && cachefile[0] != '\0'){
			// Update the cache
			FILE *fout = fopen (cachefile,"w");
			if (fout == NULL){
				fprintf (stderr,"Can't open cache file %s (%s)\n",cachefile
					,strerror (errno));
			}else{
				machineNode *pt = ret;
				while (pt != NULL){
					fprintf (fout,"%s\t%s\n",pt->workgroupName
						,pt->machineName);
					pt = pt->next;
				}
				fclose (fout);
			}
		}
	}
	for (int i=0; i<nb; i++) free ((char*)tb[i]);
	return ret;
}


int main (int argc, char *argv[])
{
	if ((argc == 2 || argc == 3) && strcmp(argv[1],"--list")==0){
		uid_t uid = getuid();
		gid_t gid = getgid();
		if (argc == 2){
			// we want to full map group/machine
			// litefsd-browse is smart enough to organise the info
			// in directory and subdirectory
			machineNode *root = v_scan ();
			printf ("t 10\n");
			machineNode *pt = root;
			while (pt != NULL){
				printf ("d %s/%s %d %d 1000\n",pt->workgroupName
					,pt->machineName,uid,gid);
				pt = pt->next;
			}
			net_delnode (root);
		}else if (argc == 3){
			char group[PATH_MAX];
			strncpy (group,argv[2],sizeof(group)-1);
			group[sizeof(group)-1] = '\0';
			char *pt = strchr(group,'/');
			if (pt == NULL){
				// We want the machine member of a work group
				machineNode *root = v_scan ();
				printf ("t 300\n");
				machineNode *pt = root;
				while (pt != NULL){
					if (strcasecmp(pt->workgroupName,group)==0){
						printf ("d %s %d %d 1000\n",pt->machineName,uid,gid);
					}
					pt = pt->next;
				}
				net_delnode (root);
			}else{
				*pt++ = '\0';
				char cmd[PATH_MAX];
				snprintf (cmd,sizeof(cmd)-1,"smbclient -W \"%s\" -L \"%s\""
					,group,pt);
				int fd = open ("/dev/tty",O_RDONLY);
				if (fd != -1){
					ioctl (fd,TIOCNOTTY);
					close (fd);
				}
				int tbpipe[2];
				if (pipe(tbpipe)!=-1){
					int oldout = dup(1);
					dup2 (tbpipe[1],1);
					close (tbpipe[1]);
					int err = open ("/dev/null",O_WRONLY);
					dup2 (err,2);
					close (err);
					FILE *fout = popen (cmd,"w");
					if (fout != NULL){
						dup2 (oldout,1);
						close (oldout);
						fputs ("\n",fout);
						fflush (fout);
						FILE *fin = fdopen (tbpipe[0],"r");
						char buf[1000];
						int state = 0;
						printf ("t 30\n");
						while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
							char *pt = buf;
							while (isspace (*pt)) pt++;
							if (strncmp(pt,"---------",9)==0){
								state ++;
							}else if (state == 1){
								if (pt[0] == '\0') break;
								char *start = pt;
								while (*pt > ' ') pt++;
								*pt = '\0';
								printf ("d %s %d %d 1000\n",start,uid,gid);
							}
						}
						fclose (fin);
						close (tbpipe[0]);
						pclose (fout);
					}
				}
			}
		}
	}
	return 0;
}
